@@ -1,12 +1,12 @@
 #!/usr/bin/env python
 
-#    Copyright 2016 - 2019 Alexey Stepanov aka penguinolog
+#    Copyright 2016 - 2021 Alexey Stepanov aka penguinolog
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
-#
+
 #         http://www.apache.org/licenses/LICENSE-2.0
-#
+
 #    Unless required by applicable law or agreed to in writing, software
 #    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
@@ -15,9 +15,12 @@
 
 """Property with logging on successful get/set/delete or failure."""
 
+from __future__ import annotations
+
 __all__ = ("LogOnAccess",)
 
 # Standard Library
+import inspect
 import logging
 import os
 import sys
@@ -27,9 +30,13 @@ import warnings
 
 _LOGGER: logging.Logger = logging.getLogger(__name__)
 _CURRENT_FILE = os.path.abspath(__file__)
+_OwnerT = typing.TypeVar("_OwnerT")
+_ReturnT = typing.TypeVar("_ReturnT")
+
+VALID_LOGGER_NAMES = ("LOGGER", "LOG", "logger", "log", "_logger", "_log")
 
 
-class LogOnAccess(property):
+class LogOnAccess(property, typing.Generic[_OwnerT, _ReturnT]):
     """Property with logging on successful get/set/delete or failure.
 
     .. versionadded:: 2.1.0
@@ -38,33 +45,33 @@ class LogOnAccess(property):
 
     def __init__(
         self,
-        fget: typing.Optional[typing.Callable[[typing.Any], typing.Any]] = None,
-        fset: typing.Optional[typing.Callable[[typing.Any, typing.Any], None]] = None,
-        fdel: typing.Optional[typing.Callable[[typing.Any], None]] = None,
-        doc: typing.Optional[str] = None,
+        fget: typing.Callable[[_OwnerT], _ReturnT] | None = None,
+        fset: typing.Callable[[_OwnerT, _ReturnT], None] | None = None,
+        fdel: typing.Callable[[_OwnerT], None] | None = None,
+        doc: str | None = None,
         *,
         # Extended settings start
-        logger: typing.Optional[typing.Union[logging.Logger, str]] = None,
+        logger: logging.Logger | str | None = None,
         log_object_repr: bool = True,
         log_level: int = logging.DEBUG,
         exc_level: int = logging.DEBUG,
         log_success: bool = True,
         log_failure: bool = True,
         log_traceback: bool = True,
-        override_name: typing.Optional[str] = None,
+        override_name: str | None = None,
     ) -> None:
         """Advanced property main entry point.
 
         :param fget: normal getter.
-        :type fget: typing.Optional[typing.Callable[[typing.Any, ], typing.Any]]
+        :type fget: typing.Callable[[_OwnerT], _ReturnT] | None
         :param fset: normal setter.
-        :type fset: typing.Optional[typing.Callable[[typing.Any, typing.Any], None]]
+        :type fset: typing.Callable[[_OwnerT, _ReturnT], None] | None
         :param fdel: normal deleter.
-        :type fdel: typing.Optional[typing.Callable[[typing.Any, ], None]]
+        :type fdel: typing.Callable[[_OwnerT], None] | None
         :param doc: docstring override
-        :type doc: typing.Optional[str]
+        :type doc: str | None
         :param logger: logger instance or name to use as override
-        :type logger: typing.Optional[typing.Union[logging.Logger, str]]
+        :type logger: logging.Logger | str | None
         :param log_object_repr: use `repr` over object to describe owner if True else owner class name and id
         :type log_object_repr: bool
         :param log_level: log level for successful operations
@@ -78,7 +85,7 @@ class LogOnAccess(property):
         :param log_traceback: Log traceback on exceptions
         :type log_traceback: bool
         :param override_name: override property name if not None else use getter/setter/deleter name
-        :type override_name: typing.Optional[str]
+        :type override_name: str | None
 
         Usage examples:
 
@@ -170,7 +177,7 @@ class LogOnAccess(property):
         super().__init__(fget=fget, fset=fset, fdel=fdel, doc=doc)
 
         if logger is None or isinstance(logger, logging.Logger):
-            self.__logger: typing.Optional[logging.Logger] = logger
+            self.__logger: logging.Logger | None = logger
         else:
             self.__logger = logging.getLogger(logger)
 
@@ -180,21 +187,27 @@ class LogOnAccess(property):
         self.__log_success: bool = log_success
         self.__log_failure: bool = log_failure
         self.__log_traceback: bool = log_traceback
-        self.__override_name: typing.Optional[str] = override_name
+        self.__override_name: str | None = override_name
         self.__name: str = ""
-        self.__owner: typing.Optional[type] = None
+        self.__owner: type[_OwnerT] | None = None
 
-    def __set_name__(self, owner: typing.Optional[type], name: str) -> None:
-        """Set __name__ and __objclass__ property."""
+    def __set_name__(self, owner: type[_OwnerT] | None, name: str) -> None:
+        """Set __name__ and __objclass__ property.
+
+        :param owner: owner class, where descriptor applied
+        :type owner: type[_OwnerT] | None
+        :param name: descriptor name
+        :type name: str
+        """
         self.__owner = owner
         self.__name = name
 
     @property
-    def __objclass__(self) -> typing.Optional[type]:  # pragma: no cover
+    def __objclass__(self) -> type[_OwnerT] | None:  # pragma: no cover
         """Read-only owner.
 
         :return: property owner class
-        :rtype: typing.Optional[type]
+        :rtype: type[_OwnerT] | None
         """
         return self.__owner
 
@@ -209,19 +222,19 @@ class LogOnAccess(property):
             return ""
         exc_info = sys.exc_info()
         stack: traceback.StackSummary = traceback.extract_stack()
-        full_tb: typing.List[traceback.FrameSummary] = [elem for elem in stack if elem.filename != _CURRENT_FILE]
-        exc_line: typing.List[str] = traceback.format_exception_only(*exc_info[:2])
+        full_tb: list[traceback.FrameSummary] = [elem for elem in stack if elem.filename != _CURRENT_FILE]
+        exc_line: list[str] = traceback.format_exception_only(*exc_info[:2])
         # Make standard traceback string
         tb_text = "\nTraceback (most recent call last):\n" + "".join(traceback.format_list(full_tb)) + "".join(exc_line)
         return tb_text
 
-    def __get_obj_source(self, instance: typing.Any, owner: typing.Optional[type] = None) -> str:
+    def __get_obj_source(self, instance: _OwnerT, owner: type[_OwnerT] | None = None) -> str:
         """Get object repr block.
 
         :param instance: object instance
-        :type instance: typing.Any
+        :type instance: _OwnerT
         :param owner: object class (available for getter usage only)
-        :type owner: typing.Optional[type]
+        :type owner: type[_OwnerT] | None
         :return: repr of object if it not disabled else repr placeholder
         :rtype: str
         """
@@ -233,29 +246,39 @@ class LogOnAccess(property):
             return f"<{self.__objclass__.__name__}() at 0x{id(instance):X}>"
         return f"<{instance.__class__.__name__}() at 0x{id(instance):X}>"
 
-    def _get_logger_for_instance(self, instance: typing.Any) -> logging.Logger:
+    def _get_logger_for_instance(self, instance: _OwnerT) -> logging.Logger:
         """Get logger for log calls.
 
         :param instance: Owner class instance. Filled only if instance created, else None.
-        :type instance: typing.Optional[owner]
+        :type instance: _OwnerT
         :return: logger instance
         :rtype: logging.Logger
         """
-        if self.logger is not None:  # pylint: disable=no-else-return
+        if self.logger is not None:
             return self.logger
-        elif hasattr(instance, "logger") and isinstance(instance.logger, logging.Logger):
-            return instance.logger
-        elif hasattr(instance, "log") and isinstance(instance.log, logging.Logger):
-            return instance.log
+        for logger_name in VALID_LOGGER_NAMES:
+            logger_candidate = getattr(instance, logger_name, None)
+            if isinstance(logger_candidate, logging.Logger):
+                return logger_candidate
+        instance_module = inspect.getmodule(instance)
+        for logger_name in VALID_LOGGER_NAMES:
+            logger_candidate = getattr(instance_module, logger_name, None)
+            if isinstance(logger_candidate, logging.Logger):
+                return logger_candidate
         return _LOGGER
 
     @typing.overload
-    def __get__(self, instance: None, owner: typing.Optional[type] = None) -> typing.NoReturn:
+    def __get__(
+        self,
+        instance: None,
+        owner: type[_OwnerT] | None = None,
+    ) -> typing.NoReturn:
         """Get descriptor.
 
         :param instance: Owner class instance. Filled only if instance created, else None.
-        :type instance: typing.Optional[owner]
+        :type instance: None
         :param owner: Owner class for property.
+        :type owner: type[_OwnerT] | None
         :return: getter call result if getter presents
         :rtype: typing.Any
         :raises AttributeError: Getter is not available
@@ -263,24 +286,34 @@ class LogOnAccess(property):
         """
 
     @typing.overload
-    def __get__(self, instance: typing.Any, owner: typing.Optional[type] = None) -> typing.Any:  # noqa: F811
+    def __get__(
+        self,
+        instance: _OwnerT,
+        owner: type[_OwnerT] | None = None,
+    ) -> _ReturnT:  # noqa: F811
         """Get descriptor.
 
         :param instance: Owner class instance. Filled only if instance created, else None.
-        :type instance: typing.Optional[owner]
+        :type instance: _OwnerT
         :param owner: Owner class for property.
+        :type owner: type[_OwnerT] | None
         :return: getter call result if getter presents
         :rtype: typing.Any
         :raises AttributeError: Getter is not available
         :raises Exception: Something goes wrong
         """
 
-    def __get__(self, instance: typing.Any, owner: typing.Optional[type] = None) -> typing.Any:  # noqa: F811
+    def __get__(
+        self,
+        instance: _OwnerT | None,
+        owner: type[_OwnerT] | None = None,
+    ) -> _ReturnT:  # noqa: F811
         """Get descriptor.
 
         :param instance: Owner class instance. Filled only if instance created, else None.
-        :type instance: typing.Optional[owner]
+        :type instance: _OwnerT | None
         :param owner: Owner class for property.
+        :type owner: type[_OwnerT] | None
         :return: getter call result if getter presents
         :rtype: typing.Any
         :raises AttributeError: Getter is not available
@@ -296,17 +329,17 @@ class LogOnAccess(property):
             result = super().__get__(instance, owner)
             if self.log_success:
                 logger.log(self.log_level, f"{source}.{self.__name__} -> {result!r}")
-            return result
+            return result  # type: ignore[no-any-return]
         except Exception:
             if self.log_failure:
                 logger.log(self.exc_level, f"Failed: {source}.{self.__name__}{self.__traceback}", exc_info=False)
             raise
 
-    def __set__(self, instance: typing.Any, value: typing.Any) -> None:
+    def __set__(self, instance: _OwnerT, value: _ReturnT) -> None:
         """Set descriptor.
 
         :param instance: Owner class instance. Filled only if instance created, else None.
-        :type instance: typing.Optional
+        :type instance: _OwnerT
         :param value: Value for setter
         :raises AttributeError: Setter is not available
         :raises Exception: Something goes wrong
@@ -328,11 +361,11 @@ class LogOnAccess(property):
                 )
             raise
 
-    def __delete__(self, instance: typing.Any) -> None:
+    def __delete__(self, instance: _OwnerT) -> None:
         """Delete descriptor.
 
         :param instance: Owner class instance. Filled only if instance created, else None.
-        :type instance: typing.Optional
+        :type instance: _OwnerT
         :raises AttributeError: Deleter is not available
         :raises Exception: Something goes wrong
         """
@@ -352,20 +385,20 @@ class LogOnAccess(property):
             raise
 
     @property
-    def logger(self) -> typing.Optional[logging.Logger]:
+    def logger(self) -> logging.Logger | None:
         """Logger instance to use as override.
 
         :return: logger instance if set
-        :rtype: typing.Optional[logging.Logger]
+        :rtype: logging.Logger | None
         """
         return self.__logger
 
     @logger.setter
-    def logger(self, logger: typing.Union[logging.Logger, str, None]) -> None:
+    def logger(self, logger: logging.Logger | str | None) -> None:
         """Logger instance to use as override.
 
         :param logger: logger instance, logger name or None if override disable required
-        :type logger: typing.Union[logging.Logger, str, None]
+        :type logger: logging.Logger | str | None
         """
         if logger is None or isinstance(logger, logging.Logger):
             self.__logger = logger
@@ -481,20 +514,20 @@ class LogOnAccess(property):
         self.__log_traceback = value
 
     @property
-    def override_name(self) -> typing.Optional[str]:
+    def override_name(self) -> str | None:
         """Override property name if not None else use getter/setter/deleter name.
 
         :return: property name override
-        :rtype: typing.Optional[str]
+        :rtype: str | None
         """
         return self.__override_name
 
     @override_name.setter
-    def override_name(self, name: typing.Optional[str]) -> None:
+    def override_name(self, name: str | None) -> None:
         """Override property name if not None else use getter/setter/deleter name.
 
         :param name: property name override
-        :type name: typing.Optional[str]
+        :type name: str | None
         """
         self.__override_name = name
 
